#pragma once

#include "FileIsb.h"
#include <mpma/audio/Source.h>
#include <mpma/audio/Player.h>
#include <memory>
#include <vector>

// --

//plays the sound from a single track sample
class IsbSamplePlayer
{
public:
    IsbSamplePlayer(const IsbSoundSample &soundData);
    ~IsbSamplePlayer();

    void Play();
    void Stop();
    std::shared_ptr<AUDIO::Source> MakeSourceForSample();

private:
    const IsbSoundSample &sound;
    std::shared_ptr<AUDIO::Source> playerSource;
    std::shared_ptr<AUDIO::Player> player;

    IsbSamplePlayer& operator=(const IsbSamplePlayer &notImplemented);
};

// --

struct IcbSqobTheme
{
    const IcbSqobStat *Sqob;
    std::vector<const IsbSoundSample*> Tracks;
};

//helper to mix a set of audio sources together
class SourceCombiner: public AUDIO::Source
{
public:
    SourceCombiner(std::vector<std::shared_ptr<AUDIO::Source>> sourcesToCombine);

    virtual nuint FillData(void *data, nuint maxSamples);

    nuint GetRemainingSamples() const;
    inline bool IsStereo() const { return true; }
    nuint GetSampleRate() const;

    virtual void Seek(nuint sampleNumber);

private:
    std::vector<std::shared_ptr<Source>> sources;

    std::vector<sint16> tempSamplesSingle;
    std::vector<sint32> tempSamplesCombine;

    bool atEnd;
};

//correlates isb and icb files and plays the resulting sqob track combinations
class IcbIsbSqobPlayer
{
public:
    IcbIsbSqobPlayer(const FileIsb &isb, const FileIsb &icb);

    std::list<IcbSqobTheme> SqobThemes;

    void Play(IcbSqobTheme *theme);
    void Stop();

    std::shared_ptr<SourceCombiner> MakeSourceForTheme(IcbSqobTheme *theme);

private:
    std::shared_ptr<AUDIO::Player> player;
    std::list<std::shared_ptr<SourceCombiner>> combiners;
};

// --

struct IcbSdriTheme
{
    const IcbSdriStat *Sdri;
    std::vector<const IcbSqobTheme*> Sqobs;
};

//helper to chain a series of audio sources together
class SourceChainer: public AUDIO::Source
{
public:
    SourceChainer(std::vector<std::shared_ptr<AUDIO::Source>> sourcesToChain);

    virtual nuint FillData(void *data, nuint maxSamples);

    nuint GetRemainingSamples() const;
    inline bool IsStereo() const { return true; }
    nuint GetSampleRate() const;

    virtual void Seek(nuint sampleNumber);

private:
    std::vector<std::shared_ptr<Source>> sources;
    nuint curSource;
    bool atEnd;
};

//uses information in sdri to chain together sqobs
class IcbIsbSdriPlayer
{
public:
    IcbIsbSdriPlayer(const FileIsb &isb, const FileIsb &icb);

    std::list<IcbSdriTheme> SdriThemes;

    void Play(IcbSdriTheme *theme);
    void Stop();

    std::shared_ptr<AUDIO::Source> MakeSourceForTheme(IcbSdriTheme *theme);

private:
    std::shared_ptr<AUDIO::Player> player;
    IcbIsbSqobPlayer sqobData;
    std::shared_ptr<AUDIO::Source> chainer;
};
