#pragma once

#include "OSFilePicker.h"
#include <Windows.h>
#include <Shlobj.h>
#include <vector>

namespace
{
    std::string lastDirectory;
}

std::string InvokeOSFilePicker(bool isSave, const std::string &typeName, const std::string &typeExt, const std::string &extToAddIfNone, const std::string &initialFilename)
{
    std::vector<char> filterString;
    filterString.insert(filterString.end(), typeName.begin(), typeName.end());
    filterString.push_back(0);
    filterString.insert(filterString.end(), typeExt.begin(), typeExt.end());
    filterString.push_back(0);
    filterString.push_back(0);

    char outFilename[MAX_PATH+1]={0};
    if (initialFilename.size()>0)
        strcpy(outFilename, initialFilename.c_str());

    OPENFILENAME of={0};
    of.lStructSize=sizeof(OPENFILENAME);
    of.lpstrFilter=&filterString[0];
    of.lpstrFile=outFilename;
    of.nMaxFile=MAX_PATH;
    of.lpstrDefExt=extToAddIfNone.c_str();
    of.Flags=OFN_NOCHANGEDIR|OFN_PATHMUSTEXIST;

    bool isOk=false;
    if (isSave)
        isOk=0!=GetSaveFileName(&of);
    else
        isOk=0!=GetOpenFileName(&of);

    if (!isOk)
        return "";
    else
        return outFilename;
}

static int CALLBACK CallbackForDirectoryPicker(HWND hwnd, UINT uMsg, LPARAM, LPARAM lpData)
{
    if (uMsg==BFFM_INITIALIZED && ((char*)lpData)[0]!=0)
        SendMessage(hwnd, BFFM_SETSELECTION, true, lpData);
    return 0;
}

std::string InvokeOSDirectoryPicker()
{
    BROWSEINFO bi={0};
    bi.ulFlags=BIF_USENEWUI;
    bi.lParam=(LPARAM)lastDirectory.c_str();
    bi.lpfn=CallbackForDirectoryPicker;

    ITEMIDLIST *ret=SHBrowseForFolder(&bi);
    if (!ret)
        return std::string();

    std::string directory;
    char temp[MAX_PATH]={0};
    if (SHGetPathFromIDList(ret, temp))
        directory=temp;

    IMalloc *wtf=nullptr;
    if (SUCCEEDED(SHGetMalloc(&wtf)))
    {
        wtf->Free(ret);
        wtf->Release();
        wtf=nullptr;
    }

    if (!directory.empty())
        lastDirectory=directory;

    return directory;
}

void InvokeOSMessageBox(const std::string &message, bool isError)
{
    MessageBox(0, message.c_str(), isError?"Error":"Message", (isError?MB_ICONERROR:MB_ICONINFORMATION)|MB_OK);
}