#include <mpma/gfxsetup/GFXSetup.h>
#include <mpma/gfx/TextWriter.h>
#include <mpma/input/Mouse.h>
#include <GL/glu.h>
#include <sstream>

#include "ProgressStream.h"

nuint ProgressStream::FillData(void *data, nuint maxSamples)
{
    if (!source)
        return 0;

    nuint amount=source->FillData(data, maxSamples);
    UpdateProgress(amount);
    return amount;
}

nuint ProgressStream::GetRemainingSamples() const
{
    if (!source)
        return 0;

    return source->GetRemainingSamples();
}

bool ProgressStream::IsStereo() const
{
    if (!source)
        return true;

    return source->IsStereo();
}

nuint ProgressStream::GetSampleRate() const
{
    if (!source)
        return 44100;

    return source->GetSampleRate();
}

void ProgressStream::Seek(nuint sampleNumber)
{
    //not seekable
}

void ProgressStream::SetNewSource(const std::string &newLabel, std::shared_ptr<AUDIO::Source> passthroughSource)
{
    if (!currentLabel.empty())
    {
        std::stringstream ss;
        ss<<currentLabel<<": Done";
        completedLabels.push_back(ss.str());
    }

    source=passthroughSource;
    currentLabel=newLabel;
    currentProgress=0;
    timer.Step(true);

    UpdateProgress(0);
}

void ProgressStream::UpdateProgress(nuint amount)
{
    bool doRender=false;
    if (timer.Step(false)>0.1f || currentProgress==0)
    {
        doRender=true;
        timer.Step(true);
    }

    currentProgress+=amount;

    if (!doRender)
        return;

    const GFX::GraphicsSetup *windowState=GFX::GetWindowState();
    if (!windowState)
    {
        source.reset();
        return;
    }

    //setup transform
    glViewport(0, 0, windowState->Width, windowState->Height);

    glClearColor(0.0f, 0.0f, 0.0f, 1);
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
    glDisable(GL_DEPTH_TEST);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluOrtho2D(0, windowState->Width, 0, windowState->Height);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

    //draw title
    GFX::TextWriter titleWriter;
    titleWriter.FontSize=24;
    titleWriter.Text<<title;
    titleWriter.CreateTextImage();
    titleWriter.RenderTexture((float)(int)(windowState->Width/2.0f-titleWriter.GetTextWidth()/2.0f), (float)(int)(windowState->Height-5.0f));

    //draw progress
    GFX::TextWriter progressWriter;
    for (std::string &l:completedLabels)
        progressWriter.Text<<l<<"\n";
    if (!currentLabel.empty())
        progressWriter.Text<<currentLabel<<": Audio samples rendered: "<<currentProgress;
    progressWriter.CreateTextImage();
    progressWriter.ViewWindowHeight=windowState->Height-100;
    progressWriter.ViewWindowScroll=progressWriter.GetMaxViewWindowScroll();
    progressWriter.RenderTexture(10.0f, windowState->Height-50.0f);

    //emit frame
    GFX::UpdateWindow();
}
