#include <mpma/gfxsetup/GFXSetup.h>
#include <mpma/gfx/TextWriter.h>
#include <mpma/input/Mouse.h>
#include <GL/glu.h>

#include "TextMenu.h"

TextMenuEntry::TextMenuEntry(const std::string &text, const std::string paramString, int paramInt, void *paramPtr)
{
    Text=text;
    ParamString=paramString;
    ParamInt=paramInt;
    ParamPtr=paramPtr;

    textWriter.FontSize=14;
}

void TextMenuEntry::ReRenderText()
{
    textWriter.Text.Clear();
    if (Active)
        textWriter.Text.AppendColorChange(224, 255, 208);
    else
        textWriter.Text.AppendColorChange(192, 192, 192);
    textWriter.Text<<Text;

    textWriter.CreateTextImage();
    textWriter.CreateTexture();
}

TextMenuEntry* TextMenu::GetChoice()
{
    while (true) //if we loop back it means we scrolled and have a new offset for everything
    {
        const GFX::GraphicsSetup *windowState=GFX::GetWindowState();
        if (!windowState)
            return 0;

        //prepare title text
        titleWriter.Text.Clear();
        titleWriter.Text.AppendColorChange(255, 255, 255);
        titleWriter.Text<<Title;
        titleWriter.CreateTextImage();
        titleWriter.CreateTexture();

        //precompute text images for the scrollable choices and figure out the column width we need
        int columnWidth=30;
        for (auto i=Choices.begin(); i!=Choices.end(); ++i)
        {
            i->ReRenderText();

            if ((int)i->textWriter.GetTextWidth()>columnWidth)
                columnWidth=i->textWriter.GetTextWidth();
        }

        columnWidth+=10;

        //arrange positions of choices
        const float dividerBarY=45.0f;
        const int minX=5;
        const int maxX=windowState->Width-5;
        const int alwaysChoicesY=30;
        int x=minX;
        const int ySpacing=18;
        const int maxY=730;
    
        for (auto i=AlwaysVisibleChoices.begin(); i!=AlwaysVisibleChoices.end(); ++i)
        {
            i->clickingSetsAsActive=false;
            i->ReRenderText();

            i->xMin=x;
            i->xMax=x+i->textWriter.GetTextWidth();
            i->yMin=alwaysChoicesY;
            i->yMax=alwaysChoicesY+i->textWriter.GetTextHeight();

            x+=i->textWriter.GetTextWidth()+40;
        }

        const int choicesMinY=50;
        int y=50;
        x=minX;
        y=choicesMinY;

        int columnsNeeded=1;
        for (auto i=Choices.begin(); i!=Choices.end(); ++i)
        {
            i->xMin=x+xOffset;
            i->xMax=x+i->textWriter.GetTextWidth()+xOffset;
            i->yMin=y;
            i->yMax=y+i->textWriter.GetTextHeight();

            y+=ySpacing;
            if (y>maxY)
            {
                y=choicesMinY;
                x+=columnWidth;
                ++columnsNeeded;
            }
        }

        //combine choices into a single list
        std::vector<TextMenuEntry*> combinedChoices;
        combinedChoices.reserve(Choices.size()+AlwaysVisibleChoices.size());

        for (auto i=AlwaysVisibleChoices.begin(); i!=AlwaysVisibleChoices.end(); ++i)
        {
            combinedChoices.push_back(&*i);
        }

        for (auto i=Choices.begin(); i!=Choices.end(); ++i)
        {
            combinedChoices.push_back(&*i);
        }

        //add in scrolling options if the list is too big
        TextMenuEntry tmeScrollLeft("(Scroll Left)", "", 0, 0);
        tmeScrollLeft.ReRenderText();
        tmeScrollLeft.xMin=5;
        tmeScrollLeft.xMax=5+tmeScrollLeft.textWriter.GetTextWidth();
        tmeScrollLeft.yMin=5;
        tmeScrollLeft.yMax=5+tmeScrollLeft.textWriter.GetTextHeight();

        TextMenuEntry tmeScrollRight("(Scroll Right)", "", 0, 0);
        tmeScrollRight.ReRenderText();
        tmeScrollRight.xMin=windowState->Width-5-tmeScrollRight.textWriter.GetTextWidth();
        tmeScrollRight.xMax=windowState->Width-5;
        tmeScrollRight.yMin=5;
        tmeScrollRight.yMax=5+tmeScrollRight.textWriter.GetTextHeight();

        int columnsAvailable=(maxX-minX)/columnWidth;
        if (columnsNeeded>columnsAvailable)
        {
            combinedChoices.push_back(&tmeScrollLeft);
            combinedChoices.push_back(&tmeScrollRight);
        }

        //draw and pick
        INPUT::ClearState();
        while (true)
        {
            windowState=GFX::GetWindowState();
            if (!windowState)
                return 0;

            //handle mouse picks
            int scrollAmount=0;

            if (INPUT::MOUSE::GetWheelDirection()<0)
            {
                scrollAmount=-50;
            }
            else if (INPUT::MOUSE::GetWheelDirection()>0)
            {
                scrollAmount=+50;
            }
            else if (INPUT::MOUSE::IsButtonNewlyPressed(INPUT::MOUSE::LEFT_BUTTON))
            {
                INPUT::MOUSE::MousePixelPosition mpos=INPUT::MOUSE::GetPixelPosition();
                mpos.Y=(sint16)windowState->Height-mpos.Y;

                for (auto i=combinedChoices.begin(); i!=combinedChoices.end(); ++i)
                {
                    TextMenuEntry &tme=**i;

                    if (mpos.X>=tme.xMin && mpos.X<=tme.xMax && mpos.Y>=tme.yMin && mpos.Y<=tme.yMax)
                    {
                        if (&tme==&tmeScrollLeft)
                        {
                            scrollAmount=-100;
                        }
                        else if (&tme==&tmeScrollRight)
                        {
                            scrollAmount=100;
                        }
                        else
                        {
                            if (tme.clickingSetsAsActive)
                            {
                                //mark only this one as active
                                for (auto h=combinedChoices.begin(); h!=combinedChoices.end(); ++h)
                                    (**h).Active=false;
                                tme.Active=true;
                            }

                            //clear window and return the choice
                            glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
                            GFX::UpdateWindow();

                            return &tme;
                        }
                    }
                }
            }

            if (columnsNeeded>columnsAvailable)
            {
                xOffset-=scrollAmount;
                if (-xOffset<0)
                    xOffset=0;
                if (-xOffset>(columnsNeeded-columnsAvailable)*columnWidth)
                    xOffset=-(columnsNeeded-columnsAvailable)*columnWidth;
            }

            //setup transform
            glViewport(0, 0, windowState->Width, windowState->Height);

            glClearColor(0.0f, 0.0f, 0.0f, 1);
            glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
            glDisable(GL_DEPTH_TEST);

            glMatrixMode(GL_PROJECTION);
            glLoadIdentity();
            gluOrtho2D(0, windowState->Width, 0, windowState->Height);

            glMatrixMode(GL_MODELVIEW);
            glLoadIdentity();

            glEnable(GL_BLEND);
            glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

            //draw title
            titleWriter.RenderTexture((float)(int)(windowState->Width/2.0f-titleWriter.GetTextWidth()/2.0f), (float)(int)(windowState->Height-5.0f));

            //draw text
            for (auto i=combinedChoices.begin(); i!=combinedChoices.end(); ++i)
                (**i).textWriter.RenderTexture((float)(**i).xMin, (float)windowState->Height-(**i).yMin);

            //draw seperator between variable choices and always visible choices
            glBegin(GL_TRIANGLE_FAN);
            glColor4f(0.4f, 0.4f, 0.4f, 1.0f);
            glVertex4f(0.0f,                      windowState->Height-dividerBarY,      0.0f, 1.0f);
            glVertex4f((float)windowState->Width, windowState->Height-dividerBarY,      0.0f, 1.0f);
            glVertex4f((float)windowState->Width, windowState->Height-dividerBarY-1.0f, 0.0f, 1.0f);
            glVertex4f(0.0f,                      windowState->Height-dividerBarY-1.0f, 0.0f, 1.0f);
            glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
            glEnd();

            //next frame
            GFX::UpdateWindow();
            MPMA::Sleep(33);

            if (scrollAmount) //can't break to the outter loop to refigure things out until we update window so input gets flushed
                break;
        }
    }
}

TextMenu::TextMenu()
{
    titleWriter.FontSize=14;
}
